package org.bench4q.agent.scenario.node;

import java.util.ArrayList;
import java.util.Stack;

import org.bench4q.agent.datacollector.DataCollector;
import org.bench4q.agent.scenario.Parameter;
import org.bench4q.agent.scenario.instruction.Instruction;
import org.bench4q.agent.scenario.instruction.TestInstruction;
import org.bench4q.share.models.agent.ParameterModel;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.ConditionBehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.IfBehaviorModel;
import org.bench4q.share.models.agent.statistics.BehaviorBriefModel;

/**
 * 
 * @author coderfengyun
 *
 */
public abstract class Behavior {
	public static final String CONTROL_BEHAVIOR = "CONTROLBEHAVIOR";
	public static final String USER_BEHAVIOR = "USERBEHAVIOR";
	public static final String TIMER_BEHAVIOR = "TIMERBEHAVIOR";
	public static final String TEST_BEHAVIOR = "TESTBEHAVIOR";

	private int id;
	private String use;
	private String name;
	private Parameter[] parameters;

	protected Behavior() {
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getUse() {
		return use;
	}

	public void setUse(String use) {
		this.use = use;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Parameter[] getParameters() {
		return parameters;
	}

	public void setParameters(Parameter[] parameters) {
		this.parameters = parameters;
	}

	public abstract boolean shouldBeCount();

	public abstract BehaviorBriefModel getBehaviorBriefResult(
			DataCollector dataStatistics);

	public String getSpecificParamValue(String paramName) {
		for (Parameter parameter : this.getParameters()) {
			if (parameter.getKey().equalsIgnoreCase(paramName)) {
				return parameter.getValue();
			}
		}
		return "";
	}

	/**
	 * No matter which one to inherit from this class, it should call
	 * baseCompile in its own compile function.
	 * 
	 * This function has side effect, the result is both code and conditions
	 */

	public abstract void compile(ArrayList<Instruction> code,
			Stack<TestInstruction> conditions);

	public static Behavior buildWith(BehaviorModel behaviorModel) {
		Behavior behavior = null;
		if (behaviorModel.getType().equalsIgnoreCase(TIMER_BEHAVIOR)) {
			behavior = new TimerBehavior(behaviorModel);
		} else if (behaviorModel.getType().equalsIgnoreCase(USER_BEHAVIOR)) {
			behavior = new UserBehavior(behaviorModel);
		} else if (behaviorModel.getType().equalsIgnoreCase(CONTROL_BEHAVIOR)) {
			behavior = new ControlBehavior(behaviorModel);
		} else if (behaviorModel.getType().equalsIgnoreCase(TEST_BEHAVIOR)) {
			// TODO: refactor this
			if (behaviorModel instanceof IfBehaviorModel) {
				behavior = new IfBehavior((IfBehaviorModel) behaviorModel);
			} else if (behaviorModel instanceof ConditionBehaviorModel) {
				behavior = new ConditionBehavior(
						(ConditionBehaviorModel) behaviorModel);
			} else {
				throw new RuntimeException("Error type: TestInstruction"
						+ behaviorModel.getClass().getName());
			}

		} else {
			throw new IllegalArgumentException(
					"The input BehaviorModel's type is not proper");
		}
		return behavior;
	}

	protected void buildAndGenerateParameterPart(BehaviorModel behaviorModel) {
		this.setName(behaviorModel.getName());
		this.setUse(behaviorModel.getUse());
		this.setId(behaviorModel.getId());
		this.setParameters(new Parameter[behaviorModel.getParameters().size()]);

		for (int k = 0; k < behaviorModel.getParameters().size(); k++) {
			ParameterModel parameterModel = behaviorModel.getParameters()
					.get(k);
			this.getParameters()[k] = Parameter.generateParamParts(
					parameterModel.getKey(), parameterModel.getValue());
		}
	}
}
